/***
    Copyright (C) 2011-2012 Avi Romanoff <aviromanoff@gmail.com>
    Copyrigth (C) 2013 elementary, LLC

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
***/

/**
 * Tools and classes for interacting with the Switchboard application,
 * and building plugs for it.
 *
 */
namespace Pantheon.Switchboard {
    /**
     * D-Bus interface to interact with the Switchboard app.
     *
     *
     */
    [DBus (name = "org.elementary.switchboard")]
    public interface SwitchboardController : GLib.Object {

        /**
         * Signal emitted when the plug is closed by the Switchboard app.
         * You shouldn't connect to this -- you should override ``on_exit``.
         */
        public signal void plug_closed ();

        /**
         * Gets the X11 Window ID (WID) of the Switchboard app.
         *
         * Call this method to get the X.Window ID of the Switchboard app.
         * You shouldn't touch this -- ``register`` handles it for yo
         *
         * @returns X.Window id of the Switchboard app.
         */
        internal abstract int get_socket_wid () throws IOError;

        /**
         * Sets the visiblity of the progress bar in the Switchboard app.
         *
         * Call this method to show or hide the progress bar.
         *
         * @param visibility ``true`` == visible, ``false`` == invisible.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract void progress_bar_set_visible (bool visibility) throws IOError;

        /**
         * Sets the fraction of the progress bar in the Switchboard app.
         *
         * Call this method to set the fraction (percent, or how full the bar is)
         * of the progress bar in the Switchboard app.
         *
         * @param fraction The fraction (0.0-1.0) to set the progress bar on the Switchboard app to.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract void progress_bar_set_fraction (double fraction) throws IOError;

        /**
         * Sets the text displayed below the progress bar in the Switchboard app.
         *
         * Call this method to set (or hide, by setting ``text`` to "") the progress bar text.
         *
         * @param text The text that will be set for the progress bar.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract void progress_bar_set_text (string text) throws IOError;

        /**
         * Pulses the progress bar in the Switchboard app.
         *
         * Call this method to pulse the progress bar (move by a small amount) in the Switchboard app.
         * Call this periodically if you have no idea at all how long a tak will take to complete.
         *
         * @deprecated 0.2
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        [Deprecated (since = "0.2", replacement = "progress_bar_set_fraction ()")]
        public abstract void progress_bar_pulse () throws IOError;

        /**
         * Emitted when Switchboard's search box becomes activated.
         *
         * Connect to this signal if you want to define what will happen
         * when the search box is clicked or selected via the keyboard.
         *
         */
        public signal void search_box_activated ();

        /**
         * Emitted when Switchboard's search box has it's text changed.
         *
         * Connect to this signal if you want to respond to changes in the
         * search box text.
         *
         */
        public signal void search_box_text_changed ();

        /**
         * Sets the sensitivity of the search box in the Switchboard app. (Whether it can be focused).
         *
         * Call this method to make the search box sensitive or insensitive.
         * The search box should be set to insensitive if the plug has no use for it.
         * If it is insensitive, the user cannot interact with it.
         *
         * @param sensitivity ``true`` == sensitive, ``false`` == insensitive.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract void search_box_set_sensitive (bool sensitivity) throws IOError;

        /**
         * Sets the placeholder text of the search box in the Switchboard app.
         *
         * Call this method to set the placeholder text (styled as per the elementary HIG).
         *
         * @param text The placeholder text to be used for the search box.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract void search_box_set_text (string text) throws IOError;

        /**
         * Gets the text of the search box in the Switchboard app.
         *
         * Call this method to get the current text in the search box in the Switchboard app.
         *
         * @return The text in the search box as a ``string``.
         * @throws GLib.IOError Couldn't talk with Switchboard.
         */
        public abstract string search_box_get_text () throws IOError;

    }

    /**
     * Abstract base class that provides convenience methods to communicate
     * with the Switchboard application and create Plugs for Switchboard. All
     * plugs should inherit from this class.
     */
    public abstract class Plug : Gtk.Plug {
        /**
         * Connects the switchboard controller up with D-Bus.
         *
         * pantheon_switchboard_plug_construct:
         * Return value: (transfer none): a plug
         */
        public Plug () {
            try {
                switchboard_controller = Bus.get_proxy_sync (BusType.SESSION, "org.elementary.switchboard",
                                                             "/org/elementary/switchboard");
            } catch (IOError e) {
                log ("Switchboard.Plug", LogLevelFlags.LEVEL_DEBUG,
                     "%s", e.message);
            }
        }

        /**
         * The name of the Plug, used only for internal debugging.
         *
         * This string is only used for debugging Switchboard and the
         * plug -- the Switchboard app never sees this variable. Rather,
         * it reads from the .plug file that is describing the plug.
         */
        public string plug_name;

        /**
         * Enables communication and interaction with the Switchboard app.
         *
         */
        public SwitchboardController switchboard_controller;

        /**
         * Registers the Plug with Switchboard and loads it into the Switchboard app.
         *
         * This method handles connects to Switchboard via D-Bus and embedding via XEMBED.
         * Don't override it unless you know //exactly// what you're doing.
         *
         * @param plug_name The identifier for the plug, purely for debugging purposes.
         * Use the .plug file to describe the plug and give it a title.
         */
        public void register (string plug_name) {
            this.plug_name = plug_name;

            try {
                base.construct (switchboard_controller.get_socket_wid ());
            } catch (IOError e) {
                log ("Switchboard.Plug", LogLevelFlags.LEVEL_DEBUG, "%s",
                     e.message);
            }

            switchboard_controller.plug_closed.connect (on_exit);
        }

        /**
         * Called when the plug is closed by Switchboard.
         *
         * Clean up code for saving plug state, etc goes here.
         * Calls ``Gtk.main_quit()``. If you override this method,
         * be sure to call it as the last thing you do.
         */
        public void on_exit () {
            log ("Switchboard.Plug", LogLevelFlags.LEVEL_INFO,
                 "This is the %s plug, signing off gracefully!\n",
                 plug_name);

            Gtk.main_quit ();
        }
    }
}
